#!/bin/bash
#

set -o nounset
set -o errexit
set -x

# -----------------------------------------------------------------------------
# Author - F. Massonnet
# Purpose- Process perturbations of DFS5.2 forcing and adds them
#          to the actual forcing. 
# What the script does:
#   1) Interpolate from daily to 3hourly
#   2) Add 29 February if necessary
#   3) Add to the true forcing
# -----------------------------------------------------------------------------

# Which variable of the DFS5.2 forcing set has to be perturbed?
# Possibilities: t2, q2, u10, v10, qlw, qsw, snow or precip
var=v10

alpha=1.0 # VERY IMPORTANT: strength of the perturbation. 1 = same as interannual variability. 0.5 = half of it.

# First and end years for which a perturbation has to be created. Must match yearb and yeare
# in the scripts preprocess.bash and mkpert.R (i.e., perturbations have to be available,
# otherwise no new forcing can be generated).
yearb=1974
yeare=1990

nmb=1 # members to loop over. nmb = first member; nme = last one.
        # Usually the first member ("fc0") is the true forcing so it should *NOT* be perturbed.
        # Hence put nmb=1 and nme=25 if you want 25 perturbed forcings
nme=24  

workdir=/esnas/scratch/$USER/TMP/TMP_v10_24685/ # Where all perturbations are recorded
                                               # This folder was defined during the execution of preprocess.bash

outtag=pDFS5.2                                  # Name of the forcing after perturbations are created
                                                # Can be the same (DFS5.2) or modified (perturbed-DFS5.2 for instance)
outdir=/esnas/releases/fg/ocean/$outtag/

cd $workdir
echo "Workdir is $workdir"

case ${var} in
  t2)
    min=100.0 	# Min and max values allowed
    max=400.0
    freq=3hour  # Frequency of availability
    ntim=2920   # Number of time steps in a year
    fvar=${var} # Name of the variable in the NetCDF
    ;;
  q2)
    min=0.0
    max=0.1
    freq=3hour
    ntim=2920
    fvar=${var}
    ;;
  u10)
    min=-100.0
    max=100.0
    freq=3hour
    ntim=2920
    fvar=${var}
    ;;
  v10)
    min=-100.0
    max=100.0
    freq=3hour
    ntim=2920
    fvar=${var}
    ;;
  qsw)
    min=0.0
    max=1000.0
    freq=1day
    ntim=365
    fvar=radsw
    ;;
  qlw)
    min=0.0
    max=1000.0
    freq=1day
    ntim=365
    fvar=radlw
    ;;
  snow)
    min=0.0
    max=0.001
    freq=1day
    ntim=365
    fvar=${var}
    ;;
  precip)
    min=0.0
    max=0.01
    freq=1day
    ntim=365
    fvar=${var}
    ;;
  *)
  echo "Variable $var unknown"
  exit
esac

for year in `seq ${yearb} ${yeare}`
do
  for mm in `seq $nmb $nme`
  do
    if [ $mm == 0 ] 
    then
      echo "WARNING!!!!"
      echo "Usually, at BSC, member 0 is booked to point towards the true forcing"
      echo "By creating a perturbed forcing named fc00, and then copying to the source"
      echo "directory, you are going to erase a symbolic link that points to the true forcing"
      echo "and thereby erase the true forcing!!"
      echo "Since this is highly dangerous, this script is aborting."
      echo "Contact francois.massonnet@bsc.es for further questions"
  
      exit
    fi
    m=$(printf "%02d" $mm)
    # There is a small trick here. If we have 3 days of data and ask for 3-hourly interpolation, we will have only 17 points and not 24.
    # We have in fact (ndays - 1) * 8 + 1. So the trick is to append the last day to the data twice and remove the last time frame.
    # 
    # Extract the last time frame
    ncks -F -O -d time,365,365 pert_${var}_DFS5.2_${year}_fc${m}.nc ./tmp.${year}.${m}.nc
    # Append it 
    ncrcat -F -O pert_${var}_DFS5.2_${year}_fc${m}.nc tmp.${year}.${m}.nc pert_${var}_DFS5.2_${year}_fc${m}.nc.1
    # Set time axis
    cdo settaxis,${year}-01-01,00:00,1day pert_${var}_DFS5.2_${year}_fc${m}.nc.1 pert_${var}_DFS5.2_${year}_fc${m}.nc.2
    # Interpolate in time
    cdo inttime,${year}-01-01,00:00,${freq} pert_${var}_DFS5.2_${year}_fc${m}.nc.2 pert_${var}_DFS5.2_${year}_fc${m}.nc.3
    # Remove the last time frame
    ncks -F -O -d time,1,${ntim} pert_${var}_DFS5.2_${year}_fc${m}.nc.3 pert_${var}_DFS5.2_${year}_fc${m}.nc.4

    # Add the desired fraction "alpha" of the perturbation to the true forcing
    cdo add -mulc,${alpha} pert_${var}_DFS5.2_${year}_fc${m}.nc.4 ${var}_DFS5.2_${year}.nc ${var}_fc${m}_DFS5.2_${year}.nc.0
    cdo setrtoc,-10000000000,${min},${min} ${var}_fc${m}_DFS5.2_${year}.nc.0 ${var}_fc${m}_DFS5.2_${year}.nc.1 
    cdo setrtoc,${max},10000000000,${max}  ${var}_fc${m}_DFS5.2_${year}.nc.1 ${outdir}/${var}_fc${m}_${outtag}_${year}.nc

    chmod 777 ${outdir}/${var}_fc${m}_${outtag}_${year}.nc

    rm -f tmp.${year}.${m}.nc pert_${var}_DFS5.2_${year}_fc${m}.nc.? ${var}_fc${m}_DFS5.2_${year}.nc.?
    done
done
